/***************************************************************************
                          Trie.hpp  -  description
                             -------------------
    begin                : cs dec 26 2002
    copyright            : (C) 2002 by Ferenc Bodon
    email                : bodon@mit.bme.hu
 ***************************************************************************/

#ifndef Trie_HPP
#define Trie_HPP

/**
  *@author Ferenc Bodon
  */

typedef unsigned long itemtype;
//typedef unsigned short itemtype;

#include <vector>
#include <set>

using namespace std;

class Apriori_Trie;

/** This class represent a general Trie.

   We can regard the trie as a recursive data structure. It has a root node and a list of (sub)trie.
   We can reach a subtree by a labeled edge (link).
   Since the root of the trie represents an itemset the counter stands for the occurrence.
   For the sake of fast traversal we also store the parent of a trie, the length of the maximal path starting from the root,
   and the edges are stored ordered according to their label.
*/

class Trie
{
friend class Apriori_Trie;

public:
   
   Trie( Trie* parent, const unsigned long init );
   
   /// It decides whether the given itemset is included in the trie or not.
   const Trie* is_included( const set<itemtype>& an_itemset, set<itemtype>::const_iterator item_it ) const;

   /// Increases the counter for those itemsets that is contained by the given basket.
   void find_candidate( const vector<itemtype>& basket, const itemtype distance_from_candidate,
                                vector<itemtype>::const_iterator it_basket, const unsigned long counter_incr=1);

   /// Deletes the tries that represent infrequent itemsets.
   void delete_infrequent( const unsigned long min_occurrence, const itemtype distance_from_candidate );

   void show_content_preorder( ) const;
   ~Trie();

private:
   /// Sets the maximal path value.
   void max_path_set( );

   /// Adds an empty state to the trie
   void add_empty_state( const itemtype item, const unsigned long init_counter=0 );

public:
   // No public members

private:
   /** itemvector stores the label of the edges.
     *
     * itemvector is always sorted!
     * itemvector[i] stores the label of the i<sup>th</sup> edge.
     * A label is a positive integer number (the code of an item).
     */
   vector<itemtype> itemvector;

   /**  statevector stores the address of the tries of the edges point to.
     *
     * statevector[i] stores address of the trie of the i<sup>th</sup> edge.
     */
   vector<Trie*> statevector;

   /// parent stores the address of the parent of the Trie.
   Trie* parent;

   /// counter stores the occurrence of the itemset represented by the Trie
   unsigned long counter;

   /// maxpath stores the length of the longest path starting from the Trie.
   itemtype maxpath;
};


#endif
